/*
 * SPDX-FileCopyrightText: Copyright 2023 Open Mobile Platform LLC <community@omp.ru>
 * SPDX-License-Identifier: BSD-3-Clause
 */
#include <camera_aurora/camera_aurora_plugin.h>
#include <flutter/method-channel.h>
#include <flutter/platform-data.h>
#include <flutter/platform-events.h>
#include <flutter/platform-methods.h>

#include <QtCore>
#include <QBuffer>
#include <QCamera>
#include <QCameraInfo>
#include <QMediaRecorder>
#include <QCameraImageCapture>

#include <unistd.h>

namespace CameraAuroraMethods
{
    constexpr auto PluginKey = "camera_aurora";

    constexpr auto AvailableCameras = "availableCameras";
    constexpr auto CreateCamera = "createCamera";
    constexpr auto Dispose = "dispose";
    constexpr auto StartCapture = "startCapture";
    constexpr auto StopCapture = "stopCapture";
    constexpr auto TakePicture = "takePicture";
    constexpr auto StartVideoRecording = "startVideoRecording";
    constexpr auto StopVideoRecording = "stopVideoRecording";
    constexpr auto PauseVideoRecording = "pauseVideoRecording";
    constexpr auto ResumeVideoRecording = "resumeVideoRecording";
}

namespace CameraAuroraEvents
{
    constexpr auto StateChanged = "cameraAuroraStateChanged";
}

CameraAuroraPlugin::CameraAuroraPlugin()
{
    PlatformEvents::SubscribeOrientationChanged([this]([[maybe_unused]] DisplayRotation orientation) {
        if (this->m_isEnableStateChanged) {
            auto state = this->m_textureCamera->GetState();
            EventChannel(CameraAuroraEvents::StateChanged, MethodCodecType::Standard).SendEvent(state);
        }
    });
}

void CameraAuroraPlugin::RegisterWithRegistrar(PluginRegistrar &registrar)
{
    m_textureCamera = new TextureCamera(registrar.GetRegisterTexture(), [this]() {
        auto state = this->m_textureCamera->GetState();
        EventChannel(CameraAuroraEvents::StateChanged, MethodCodecType::Standard).SendEvent(state);
    });

    RegisterMethods(registrar);
    RegisterEvents(registrar);
}

void CameraAuroraPlugin::RegisterMethods(PluginRegistrar &registrar)
{
    auto methods = [this](const MethodCall &call)
    {
        const auto &method = call.GetMethod();

        if (method == CameraAuroraMethods::AvailableCameras)
        {
            onAvailableCameras(call);
            return;
        }
        if (method == CameraAuroraMethods::CreateCamera)
        {
            onCreateCamera(call);
            return;
        }
        if (method == CameraAuroraMethods::Dispose)
        {
            onDispose(call);
            return;
        }
        if (method == CameraAuroraMethods::StartCapture)
        {
            onStartCapture(call);
            return;
        }
        if (method == CameraAuroraMethods::StopCapture)
        {
            onStopCapture(call);
            return;
        }

        if (method == CameraAuroraMethods::TakePicture)
        {
            onTakePicture(call);
            return;
        }
        
        if (method == CameraAuroraMethods::StartVideoRecording)
        {
            onStartVideoRecording(call);
            return;
        }
        if (method == CameraAuroraMethods::StopVideoRecording)
        {
            onStopVideoRecording(call);
            return;
        }
        if (method == CameraAuroraMethods::PauseVideoRecording)
        {
            onPauseVideoRecording(call);
            return;
        }
        if (method == CameraAuroraMethods::ResumeVideoRecording)
        {
            onResumeVideoRecording(call);
            return;
        }

        unimplemented(call);
    };

    registrar.RegisterMethodChannel(
        CameraAuroraMethods::PluginKey,
        MethodCodecType::Standard,
        methods);
}

void CameraAuroraPlugin::RegisterEvents(PluginRegistrar &registrar)
{
    registrar.RegisterEventChannel(
        CameraAuroraEvents::StateChanged, MethodCodecType::Standard,
        [this](const Encodable &)
        { this->m_isEnableStateChanged = true; return EventResponse(); },
        [this](const Encodable &)
        { this->m_isEnableStateChanged = true; return EventResponse(); });
}

/**
 * Camera
 */
void CameraAuroraPlugin::onAvailableCameras(const MethodCall &call)
{
    call.SendSuccessResponse(m_textureCamera->GetAvailableCameras());
}

void CameraAuroraPlugin::onCreateCamera(const MethodCall &call)
{
    auto cameraName = call.GetArgument<Encodable::String>("cameraName");

    auto state = m_textureCamera->Register(cameraName);

    EventChannel(CameraAuroraEvents::StateChanged, MethodCodecType::Standard).SendEvent(state);

    call.SendSuccessResponse(state);
}

void CameraAuroraPlugin::onDispose(const MethodCall &call)
{
    auto state = m_textureCamera->Unregister();

    EventChannel(CameraAuroraEvents::StateChanged, MethodCodecType::Standard).SendEvent(state);

    unimplemented(call);
}

void CameraAuroraPlugin::onStartCapture(const MethodCall &call)
{
    auto width = call.GetArgument<Encodable::Int>("width");
    auto height = call.GetArgument<Encodable::Int>("height");

    auto state = m_textureCamera->StartCapture(width, height);

    EventChannel(CameraAuroraEvents::StateChanged, MethodCodecType::Standard).SendEvent(state);

    unimplemented(call);
}

void CameraAuroraPlugin::onStopCapture(const MethodCall &call)
{
    m_textureCamera->StopCapture();

    unimplemented(call);
}

void CameraAuroraPlugin::onTakePicture(const MethodCall &call)
{
    call.SendSuccessResponse(m_textureCamera->GetImageBase64());
}

void CameraAuroraPlugin::onStartVideoRecording(const MethodCall &call)
{
    unimplemented(call);
}

void CameraAuroraPlugin::onStopVideoRecording(const MethodCall &call)
{
    unimplemented(call);
}

void CameraAuroraPlugin::onPauseVideoRecording(const MethodCall &call)
{
    unimplemented(call);
}

void CameraAuroraPlugin::onResumeVideoRecording(const MethodCall &call)
{
    unimplemented(call);
}

void CameraAuroraPlugin::unimplemented(const MethodCall &call)
{
    call.SendSuccessResponse(nullptr);
}

#include "moc_camera_aurora_plugin.cpp"
