/**
 * SPDX-FileCopyrightText: Copyright 2023 Open Mobile Platform LLC <community@omp.ru>
 * SPDX-License-Identifier: BSD-3-Clause
 */
#include <camera_aurora/texture_camera.h>
#include <camera_aurora/texture_camera_egl_helper.h>
#include <camera_aurora/texture_camera_pixels_helper.h>

#include <flutter/platform-data.h>
#include <flutter/platform-methods.h>

#include <QtCore>
#include <QBuffer>

TextureCamera::TextureCamera(TextureRegistrar *plugin, const CameraErrorHandler &onError)
    : m_plugin(plugin)
    , m_onError(onError)
    , m_variant(nullptr)
    , m_image(nullptr)
    , m_manager(StreamCameraManager())
{
    TextureCameraEGLHelper::EGLInit();
}

std::vector<Encodable> TextureCamera::GetAvailableCameras()
{
    std::vector<Encodable> cameras;
    auto count = m_manager->getNumberOfCameras();

    for (int index = 0; index < count; index++) {
        Aurora::StreamCamera::CameraInfo info;
        if (m_manager->getCameraInfo(index, info)) {
            cameras.push_back(std::map<Encodable, Encodable>{
                {"id", info.id},
                {"name", info.name},
                {"provider", info.provider},
                {"mountAngle", info.mountAngle},
            });
        }
    }

    return cameras;
}

std::string TextureCamera::GetImageBase64()
{
    if (m_image && m_camera) {
        Aurora::StreamCamera::CameraInfo info;
        if (m_camera->getInfo(info)) {
            QBuffer qbuffer;
            qbuffer.open(QIODevice::WriteOnly);
            QImage rotatedImg = m_image->transformed(QMatrix().rotate(info.mountAngle));
            rotatedImg.save(&qbuffer, "JPEG");
            return qbuffer.data().toBase64().toStdString();
        }
    }
    return "";
}

std::map<Encodable, Encodable> TextureCamera::GetState()
{
    Aurora::StreamCamera::CameraInfo info;

    if (m_camera && m_camera->getInfo(info)) {

        auto orientation = static_cast<int>(PlatformMethods::GetOrientation());

        return std::map<Encodable, Encodable>{
            {"id", info.id},
            {"textureId", m_textureId},
            {"width", m_captureWidth},
            {"height", m_captureHeight},
            {"rotationCamera", info.mountAngle},
            {"rotationDisplay", orientation},
            {"error", m_error},
        };
    }

    return std::map<Encodable, Encodable>{
        {"error", m_error}
    };
}

bool TextureCamera::CreateCamera(std::string cameraName)
{
    if (auto count = m_manager->getNumberOfCameras()) {
        for (int index = 0; index < count; index++) {
            Aurora::StreamCamera::CameraInfo info;
            if (m_manager->getCameraInfo(index, info)) {
                if (info.id == cameraName) {
                    m_camera = m_manager->openCamera(info.id);
                    if (m_camera) {
                        m_camera->setListener(this);
                        return true;
                    } else {
                        Unregister();
                        SendError("Stream camera error open camera");
                        return false;
                    }
                }
            }
        }
    }
    return false;
}

void TextureCamera::SendError(std::string error)
{
    m_error = error;
    m_onError();
}

std::map<Encodable, Encodable> TextureCamera::StartCapture(size_t width, size_t height)
{
    m_viewWidth = width;
    m_viewHeight = height;

    if (m_camera) {
        if (m_camera->captureStarted()) {
            m_camera->stopCapture();
        }
        Aurora::StreamCamera::CameraInfo info;
        if (m_camera->getInfo(info)) {
            std::vector<Aurora::StreamCamera::CameraCapability> caps;

            if (m_manager->queryCapabilities(info.id, caps)) {
                for(unsigned int i = 0; i< caps.size(); i++) {
                    if (width + height <= caps[i].width + caps[i].height || caps.size() == i - 1) {

                        m_captureWidth = caps[i].width;
                        m_captureHeight = caps[i].height;

                        if (!m_camera->startCapture(caps[i])) {
                            Unregister();
                            SendError("Stream camera error start capture");
                        }

                        break;
                    }
                }
            }
        }
    }

    return GetState();
}

void TextureCamera::StopCapture()
{
    if (m_camera && m_camera->captureStarted()) {
        m_camera->stopCapture();
    }
}

std::map<Encodable, Encodable> TextureCamera::Register(std::string cameraName)
{
    m_textureId = m_plugin->RegisterTexture(
        [this]([[maybe_unused]] size_t width, [[maybe_unused]] size_t height) {
            return m_variant;
        });

    if (CreateCamera(cameraName) && m_viewWidth != 0 && m_viewHeight != 0) {
        StartCapture(m_viewWidth, m_viewHeight);
    }

    return GetState();
}

std::map<Encodable, Encodable>  TextureCamera::Unregister()
{
    StopCapture();

    m_error = "";
    m_textureId = 0;
    m_captureWidth = 0;
    m_captureHeight = 0;
    m_variant = nullptr;
    m_camera = nullptr;

    m_plugin->UnregisterTexture(m_textureId);

    return GetState();
}

void TextureCamera::onCameraFrame(std::shared_ptr<Aurora::StreamCamera::GraphicBuffer> buffer)
{
    if (buffer->handleType == Aurora::StreamCamera::HandleType::EGL && false) {
        // @todo not tested
        auto eglImage = TextureCameraEGLHelper::EGLCreateImage(buffer);
        m_variant = new TextureVariant(FlutterEGLImage{
            eglImage,
            buffer->width,
            buffer->height,
        });
    } else {
        m_image = TextureCameraPixelsHelper::YUVtoARGB(buffer->mapYCbCr());
        auto pixels = static_cast<uint8_t *>(m_image->bits());
        m_variant = new TextureVariant(FlutterPixelBuffer{
            pixels,
            buffer->width,
            buffer->height,
        });
    }

    m_plugin->MarkTextureAvailable(m_textureId);
}

void TextureCamera::onCameraError(const std::string &errorDescription)
{
    Unregister();
    SendError(errorDescription);
}

void TextureCamera::onCameraParameterChanged([[maybe_unused]] Aurora::StreamCamera::CameraParameter parameter,
                                      const std::string &value)
{
    std::cout << "onCameraParameterChanged: " << value << std::endl; 
}
