// SPDX-FileCopyrightText: Copyright 2023 Open Mobile Platform LLC <community@omp.ru>
// SPDX-License-Identifier: BSD-3-Clause
import 'dart:convert';

import 'package:camera_platform_interface/camera_platform_interface.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import 'camera_aurora_platform_interface.dart';
import 'camera_data.dart';

enum CameraAuroraMethods {
  availableCameras,
  createCamera,
  dispose,
  startCapture,
  stopCapture,
  takePicture,
  startVideoRecording,
  stopVideoRecording,
  pauseVideoRecording,
  resumeVideoRecording,
}

enum CameraAuroraEvents {
  cameraAuroraStateChanged,
}

/// An implementation of [CameraAuroraPlatform] that uses method channels.
class MethodChannelCameraAurora extends CameraAuroraPlatform {
  /// The method channel used to interact with the native platform.
  @visibleForTesting
  final methodsChannel = const MethodChannel('camera_aurora');

  @override
  Stream<CameraState> onChangeState() async* {
    await for (final data
        in EventChannel(CameraAuroraEvents.cameraAuroraStateChanged.name)
            .receiveBroadcastStream()) {
      yield CameraState.fromJson(data);
    }
  }

  @override
  Future<List<CameraDescription>> availableCameras() async {
    final List<CameraDescription> result = [];

    final cameras = await methodsChannel.invokeMethod<List<dynamic>?>(
            CameraAuroraMethods.availableCameras.name) ??
        [];

    for (final camera in cameras) {
      final data = camera['id'].split(':');
      var direction = CameraLensDirection.external;

      if (data[1].toString().contains('rear')) {
        direction = CameraLensDirection.back;
      } else if (data[1].toString().contains('front')) {
        direction = CameraLensDirection.front;
      }

      result.add(CameraDescription(
        name: camera['id'],
        lensDirection: direction,
        sensorOrientation: camera['mountAngle'],
      ));
    }

    return result;
  }

  @override
  Future<CameraState> createCamera(String cameraName) async {
    final data = await methodsChannel
        .invokeMethod<Map<dynamic, dynamic>?>('createCamera', {
      'cameraName': cameraName,
    });
    return CameraState.fromJson(data ?? {});
  }

  @override
  Future<void> startCapture(double width, double height) async {
    await methodsChannel
        .invokeMethod<Object?>(CameraAuroraMethods.startCapture.name, {
      'width': width.round(),
      'height': height.round(),
    });
  }

  @override
  Future<void> stopCapture() async {
    await methodsChannel
        .invokeMethod<Object?>(CameraAuroraMethods.stopCapture.name, {});
  }

  @override
  Future<void> dispose() async {
    await methodsChannel
        .invokeMethod<Object?>(CameraAuroraMethods.dispose.name);
  }

  @override
  Future<XFile> takePicture(int cameraId) async {
    final image = await methodsChannel.invokeMethod<String?>(
      CameraAuroraMethods.takePicture.name,
      {'cameraId': cameraId},
    );
    final bytes = base64Decode(image!);
    return XFile.fromData(
      bytes,
      name: 'temp.jpg',
      mimeType: 'image/jpeg',
      length: bytes.length,
    );
  }

  @override
  Future<void> startVideoRecording(int cameraId,
      {Duration? maxVideoDuration}) async {
    await methodsChannel
        .invokeMethod<Object?>(CameraAuroraMethods.startVideoRecording.name, {
      'cameraId': cameraId,
    });
  }

  @override
  Future<XFile> stopVideoRecording(int cameraId) async {
    await methodsChannel
        .invokeMethod<Object?>(CameraAuroraMethods.stopVideoRecording.name, {
      'cameraId': cameraId,
    });

    // @todo Save empty data
    return XFile.fromData(
      Uint8List(0),
      name: 'file_name.mp4',

      /// @todo not set
      mimeType: 'video/mp4',
      length: 0,
    );
  }

  @override
  Future<void> pauseVideoRecording(int cameraId) async {
    await methodsChannel
        .invokeMethod<Object?>(CameraAuroraMethods.pauseVideoRecording.name, {
      'cameraId': cameraId,
    });
  }

  @override
  Future<void> resumeVideoRecording(int cameraId) async {
    await methodsChannel
        .invokeMethod<Object?>(CameraAuroraMethods.resumeVideoRecording.name, {
      'cameraId': cameraId,
    });
  }
}
